package security

import (
	"encoding/json"
	"github.com/gin-gonic/gin"
	"io/ioutil"
	"log"
	"net/http"
	"net/url"
	"os"
	"strings"
)

type keycloakUser struct {
	Active         bool   `json:"active"`
	Username       string `json:"username"`
	Email          string `json:"email"`
	ResourceAccess map[string]struct {
		Roles []string `json:"roles"`
	} `json:"resource_access"`
}

func HandleKeycloak() gin.HandlerFunc {
	validationUrl := os.Getenv("KEYCLOAK_VALIDATION_URL")
	clientId := os.Getenv("KEYCLOAK_CLIENT_ID")
	clientSecret := os.Getenv("KEYCLOAK_CLIENT_SECRET")

	return func(c *gin.Context) {
		auth := c.GetHeader("Authorization")
		auth = strings.TrimPrefix(auth, "Bearer ")
		response, err := http.PostForm(validationUrl, url.Values{
			"token":         []string{auth},
			"client_id":     []string{clientId},
			"client_secret": []string{clientSecret},
		})
		if err != nil {
			c.AbortWithStatus(http.StatusUnauthorized)
			log.Printf("couldn't validated request on keycloak server, cause: %+v", err)
			return
		}
		defer response.Body.Close()

		body, err := ioutil.ReadAll(response.Body)
		if err != nil {
			log.Printf("couldn't validated request on keycloak server, cause: %+v", err)
			c.AbortWithStatus(http.StatusUnauthorized)
			return
		}
		responseUser := keycloakUser{}
		if err := json.Unmarshal(body, &responseUser); err != nil {
			log.Printf("couldn't validated request on keycloak server, cause: %+v", err)
			c.AbortWithStatus(http.StatusUnauthorized)
			return
		}
		if !responseUser.Active {
			log.Printf("expired jwt token")
			c.AbortWithStatus(http.StatusForbidden)
			return
		}
		user := User{
			Username: responseUser.Username,
			Email:    responseUser.Email,
			roles:    responseUser.ResourceAccess[clientId].Roles,
		}
		c.Set("user", user)
		c.Next()
	}
}
